@@ -14,3 +14,11 @@ def f():
     x = 1
     y = 2
     z = x + y
+
+
+def g():
+    foo = (1, 2)
+    (a, b) = (1, 2)
+
+    bar = (1, 2)
+    (c, d) = bar
