@@ -3,7 +3,7 @@ use std::collections::BTreeSet;
 use itertools::izip;
 use rustpython_parser::ast::{
     Arg, Arguments, Cmpop, Constant, Excepthandler, ExcepthandlerKind, Expr, ExprKind, Keyword,
-    Location, Stmt, Unaryop,
+    Location, Stmt, StmtKind, Unaryop,
 };
 
 use crate::ast::operations::SourceCodeLocator;
@@ -42,18 +42,20 @@ pub fn check_not_tests(
 
     if matches!(op, Unaryop::Not) {
         if let ExprKind::Compare { ops, .. } = &operand.node {
-            match ops[..] {
-                [Cmpop::In] => {
-                    if check_not_in {
-                        checks.push(Check::new(CheckKind::NotInTest, operand.location));
+            for op in ops {
+                match op {
+                    Cmpop::In => {
+                        if check_not_in {
+                            checks.push(Check::new(CheckKind::NotInTest, operand.location));
+                        }
                     }
-                }
-                [Cmpop::Is] => {
-                    if check_not_is {
-                        checks.push(Check::new(CheckKind::NotIsTest, operand.location));
+                    Cmpop::Is => {
+                        if check_not_is {
+                            checks.push(Check::new(CheckKind::NotIsTest, operand.location));
+                        }
                     }
+                    _ => {}
                 }
-                _ => {}
             }
         }
     }
@@ -93,6 +95,46 @@ pub fn check_do_not_assign_lambda(value: &Expr, location: Location) -> Option<Ch
     }
 }
 
+fn is_ambiguous_name(name: &str) -> bool {
+    name == "l" || name == "I" || name == "O"
+}
+
+/// Check AmbiguousVariableName compliance.
+pub fn check_ambiguous_variable_name(name: &str, location: Location) -> Option<Check> {
+    if is_ambiguous_name(name) {
+        Some(Check::new(
+            CheckKind::AmbiguousVariableName(name.to_string()),
+            location,
+        ))
+    } else {
+        None
+    }
+}
+
+/// Check AmbiguousClassName compliance.
+pub fn check_ambiguous_class_name(name: &str, location: Location) -> Option<Check> {
+    if is_ambiguous_name(name) {
+        Some(Check::new(
+            CheckKind::AmbiguousClassName(name.to_string()),
+            location,
+        ))
+    } else {
+        None
+    }
+}
+
+/// Check AmbiguousFunctionName compliance.
+pub fn check_ambiguous_function_name(name: &str, location: Location) -> Option<Check> {
+    if is_ambiguous_name(name) {
+        Some(Check::new(
+            CheckKind::AmbiguousFunctionName(name.to_string()),
+            location,
+        ))
+    } else {
+        None
+    }
+}
+
 /// Check UselessObjectInheritance compliance.
 pub fn check_useless_object_inheritance(
     stmt: &Stmt,
@@ -116,9 +158,7 @@ pub fn check_useless_object_inheritance(
                             CheckKind::UselessObjectInheritance(name.to_string()),
                             expr.location,
                         );
-                        if matches!(autofix, fixer::Mode::Generate)
-                            || matches!(autofix, fixer::Mode::Apply)
-                        {
+                        if matches!(autofix, fixer::Mode::Generate | fixer::Mode::Apply) {
                             if let Some(fix) = fixes::remove_class_def_base(
                                 locator,
                                 &stmt.location,
@@ -214,9 +254,7 @@ pub fn check_assert_equals(expr: &Expr, autofix: &fixer::Mode) -> Option<Check>
             if let ExprKind::Name { id, .. } = &value.node {
                 if id == "self" {
                     let mut check = Check::new(CheckKind::NoAssertEquals, expr.location);
-                    if matches!(autofix, fixer::Mode::Generate)
-                        || matches!(autofix, fixer::Mode::Apply)
-                    {
+                    if matches!(autofix, fixer::Mode::Generate | fixer::Mode::Apply) {
                         check.amend(Fix {
                             content: "assertEqual".to_string(),
                             start: Location::new(expr.location.row(), expr.location.column() + 1),
@@ -275,7 +313,7 @@ pub fn check_repeated_keys(
                 (Some(DictionaryKey::Variable(v1)), Some(DictionaryKey::Variable(v2))) => {
                     if check_repeated_variables && v1 == v2 {
                         checks.push(Check::new(
-                            CheckKind::MultiValueRepeatedKeyVariable(v2.to_string()),
+                            CheckKind::MultiValueRepeatedKeyVariable((*v2).to_string()),
                             k2.location,
                         ))
                     }
@@ -396,6 +434,90 @@ pub fn check_literal_comparisons(
     checks
 }
 
+fn is_constant(expr: &Expr) -> bool {
+    match &expr.node {
+        ExprKind::Constant { .. } => true,
+        ExprKind::Tuple { elts, .. } => elts.iter().all(is_constant),
+        _ => false,
+    }
+}
+
+fn is_singleton(expr: &Expr) -> bool {
+    matches!(
+        expr.node,
+        ExprKind::Constant {
+            value: Constant::None | Constant::Bool(_) | Constant::Ellipsis,
+            ..
+        }
+    )
+}
+
+fn is_constant_non_singleton(expr: &Expr) -> bool {
+    is_constant(expr) && !is_singleton(expr)
+}
+
+/// Check IsLiteral compliance.
+pub fn check_is_literal(
+    left: &Expr,
+    ops: &Vec<Cmpop>,
+    comparators: &Vec<Expr>,
+    location: Location,
+) -> Vec<Check> {
+    let mut checks: Vec<Check> = vec![];
+
+    let mut left = left;
+    for (op, right) in izip!(ops, comparators) {
+        if matches!(op, Cmpop::Is | Cmpop::IsNot)
+            && (is_constant_non_singleton(left) || is_constant_non_singleton(right))
+        {
+            checks.push(Check::new(CheckKind::IsLiteral, location));
+        }
+        left = right;
+    }
+
+    checks
+}
+
+/// Check TypeComparison compliance.
+pub fn check_type_comparison(
+    ops: &Vec<Cmpop>,
+    comparators: &Vec<Expr>,
+    location: Location,
+) -> Vec<Check> {
+    let mut checks: Vec<Check> = vec![];
+
+    for (op, right) in izip!(ops, comparators) {
+        if matches!(op, Cmpop::Is | Cmpop::IsNot | Cmpop::Eq | Cmpop::NotEq) {
+            match &right.node {
+                ExprKind::Call { func, args, .. } => {
+                    if let ExprKind::Name { id, .. } = &func.node {
+                        // Ex) type(False)
+                        if id == "type" {
+                            if let Some(arg) = args.first() {
+                                // Allow comparison for types which are not obvious.
+                                if !matches!(arg.node, ExprKind::Name { .. }) {
+                                    checks.push(Check::new(CheckKind::TypeComparison, location));
+                                }
+                            }
+                        }
+                    }
+                }
+                ExprKind::Attribute { value, .. } => {
+                    if let ExprKind::Name { id, .. } = &value.node {
+                        // Ex) types.IntType
+                        if id == "types" {
+                            checks.push(Check::new(CheckKind::TypeComparison, location));
+                        }
+                    }
+                }
+                _ => {}
+            }
+        }
+    }
+
+    checks
+}
+
 /// Check TwoStarredExpressions and TooManyExpressionsInStarredAssignment compliance.
 pub fn check_starred_expressions(
     elts: &[Expr],
@@ -428,3 +550,77 @@ pub fn check_starred_expressions(
 
     None
 }
+
+/// Check BreakOutsideLoop compliance.
+pub fn check_break_outside_loop(
+    stmt: &Stmt,
+    parents: &[&Stmt],
+    parent_stack: &[usize],
+) -> Option<Check> {
+    let mut allowed: bool = false;
+    let mut parent = stmt;
+    for index in parent_stack.iter().rev() {
+        let child = parent;
+        parent = parents[*index];
+        match &parent.node {
+            StmtKind::For { orelse, .. }
+            | StmtKind::AsyncFor { orelse, .. }
+            | StmtKind::While { orelse, .. } => {
+                if !orelse.contains(child) {
+                    allowed = true;
+                    break;
+                }
+            }
+
+            StmtKind::FunctionDef { .. }
+            | StmtKind::AsyncFunctionDef { .. }
+            | StmtKind::ClassDef { .. } => {
+                break;
+            }
+            _ => {}
+        }
+    }
+
+    if !allowed {
+        Some(Check::new(CheckKind::BreakOutsideLoop, stmt.location))
+    } else {
+        None
+    }
+}
+
+/// Check ContinueOutsideLoop compliance.
+pub fn check_continue_outside_loop(
+    stmt: &Stmt,
+    parents: &[&Stmt],
+    parent_stack: &[usize],
+) -> Option<Check> {
+    let mut allowed: bool = false;
+    let mut parent = stmt;
+    for index in parent_stack.iter().rev() {
+        let child = parent;
+        parent = parents[*index];
+        match &parent.node {
+            StmtKind::For { orelse, .. }
+            | StmtKind::AsyncFor { orelse, .. }
+            | StmtKind::While { orelse, .. } => {
+                if !orelse.contains(child) {
+                    allowed = true;
+                    break;
+                }
+            }
+
+            StmtKind::FunctionDef { .. }
+            | StmtKind::AsyncFunctionDef { .. }
+            | StmtKind::ClassDef { .. } => {
+                break;
+            }
+            _ => {}
+        }
+    }
+
+    if !allowed {
+        Some(Check::new(CheckKind::ContinueOutsideLoop, stmt.location))
+    } else {
+        None
+    }
+}
