@@ -1,8 +1,9 @@
+use std::ops::Deref;
 use std::path::Path;
 
 use rustpython_parser::ast::{
     Arg, Arguments, Constant, Excepthandler, ExcepthandlerKind, Expr, ExprContext, ExprKind,
-    KeywordData, Location, Stmt, StmtKind, Suite,
+    KeywordData, Location, Operator, Stmt, StmtKind, Suite,
 };
 use rustpython_parser::parser;
 
@@ -14,6 +15,7 @@ use crate::ast::{checks, operations, visitor};
 use crate::autofix::fixer;
 use crate::checks::{Check, CheckCode, CheckKind};
 use crate::python::builtins::{BUILTINS, MAGIC_GLOBALS};
+use crate::python::future::ALL_FEATURE_NAMES;
 use crate::python::typing;
 use crate::settings::Settings;
 
@@ -24,7 +26,7 @@ struct Checker<'a> {
     locator: SourceCodeLocator<'a>,
     settings: &'a Settings,
     autofix: &'a fixer::Mode,
-    path: &'a str,
+    path: &'a Path,
     // Computed checks.
     checks: Vec<Check>,
     // Retain all scopes and parent nodes, along with a stack of indexes to track which are active
@@ -34,7 +36,8 @@ struct Checker<'a> {
     scopes: Vec<Scope>,
     scope_stack: Vec<usize>,
     dead_scopes: Vec<usize>,
-    deferred_annotations: Vec<(Location, &'a str)>,
+    deferred_string_annotations: Vec<(Location, &'a str)>,
+    deferred_annotations: Vec<(&'a Expr, Vec<usize>, Vec<usize>)>,
     deferred_functions: Vec<(&'a Stmt, Vec<usize>, Vec<usize>)>,
     deferred_lambdas: Vec<(&'a Expr, Vec<usize>, Vec<usize>)>,
     deferred_assignments: Vec<usize>,
@@ -44,13 +47,15 @@ struct Checker<'a> {
     in_literal: bool,
     seen_non_import: bool,
     seen_docstring: bool,
+    futures_allowed: bool,
+    annotations_future_enabled: bool,
 }
 
 impl<'a> Checker<'a> {
     pub fn new(
         settings: &'a Settings,
         autofix: &'a fixer::Mode,
-        path: &'a str,
+        path: &'a Path,
         content: &'a str,
     ) -> Checker<'a> {
         Checker {
@@ -64,6 +69,7 @@ impl<'a> Checker<'a> {
             scopes: vec![],
             scope_stack: vec![],
             dead_scopes: vec![],
+            deferred_string_annotations: vec![],
             deferred_annotations: vec![],
             deferred_functions: vec![],
             deferred_lambdas: vec![],
@@ -73,6 +79,8 @@ impl<'a> Checker<'a> {
             in_literal: false,
             seen_non_import: false,
             seen_docstring: false,
+            futures_allowed: true,
+            annotations_future_enabled: false,
         }
     }
 }
@@ -102,32 +110,57 @@ where
 
         // Track whether we've seen docstrings, non-imports, etc.
         match &stmt.node {
-            StmtKind::Import { .. } => {}
-            StmtKind::ImportFrom { .. } => {}
-            StmtKind::Expr { value } => {
-                if !self.seen_docstring
-                    && stmt.location.column() == 1
-                    && !operations::in_nested_block(&self.parent_stack, &self.parents)
-                {
-                    if let ExprKind::Constant {
-                        value: Constant::Str(_),
-                        ..
-                    } = &value.node
-                    {
-                        self.seen_docstring = true;
+            StmtKind::ImportFrom { module, .. } => {
+                // Allow __future__ imports until we see a non-__future__ import.
+                if self.futures_allowed {
+                    if let Some(module) = module {
+                        if module != "__future__" {
+                            self.futures_allowed = false;
+                        }
                     }
                 }
-
-                if !self.seen_non_import
-                    && stmt.location.column() == 1
+            }
+            StmtKind::Import { .. } => {
+                self.futures_allowed = false;
+            }
+            StmtKind::Expr { value } => {
+                if self.seen_docstring
+                    && !self.seen_non_import
                     && !operations::in_nested_block(&self.parent_stack, &self.parents)
                 {
                     self.seen_non_import = true;
                 }
+
+                if !self.seen_docstring
+                    && !operations::in_nested_block(&self.parent_stack, &self.parents)
+                    && matches!(
+                        &value.node,
+                        ExprKind::Constant {
+                            value: Constant::Str(_),
+                            ..
+                        },
+                    )
+                {
+                    self.seen_docstring = true;
+                }
+
+                // Allow docstrings to interrupt __future__ imports.
+                if self.futures_allowed
+                    && !matches!(
+                        &value.node,
+                        ExprKind::Constant {
+                            value: Constant::Str(_),
+                            ..
+                        },
+                    )
+                {
+                    self.futures_allowed = false;
+                }
             }
             _ => {
+                self.futures_allowed = false;
+
                 if !self.seen_non_import
-                    && stmt.location.column() == 1
                     && !operations::in_nested_block(&self.parent_stack, &self.parents)
                 {
                     self.seen_non_import = true;
@@ -151,13 +184,37 @@ where
                                 name.to_string(),
                                 Binding {
                                     kind: BindingKind::Assignment,
-                                    used: Some(global_scope_id),
+                                    used: Some((global_scope_id, stmt.location)),
                                     location: stmt.location,
                                 },
                             );
                         }
                     }
                 }
+
+                if self.settings.select.contains(&CheckCode::E741) {
+                    self.checks.extend(names.iter().filter_map(|name| {
+                        checks::check_ambiguous_variable_name(name, stmt.location)
+                    }));
+                }
+            }
+            StmtKind::Break => {
+                if self.settings.select.contains(&CheckCode::F701) {
+                    if let Some(check) =
+                        checks::check_break_outside_loop(stmt, &self.parents, &self.parent_stack)
+                    {
+                        self.checks.push(check);
+                    }
+                }
+            }
+            StmtKind::Continue => {
+                if self.settings.select.contains(&CheckCode::F702) {
+                    if let Some(check) =
+                        checks::check_continue_outside_loop(stmt, &self.parents, &self.parent_stack)
+                    {
+                        self.checks.push(check);
+                    }
+                }
             }
             StmtKind::FunctionDef {
                 name,
@@ -173,6 +230,12 @@ where
                 args,
                 ..
             } => {
+                if self.settings.select.contains(&CheckCode::E743) {
+                    if let Some(check) = checks::check_ambiguous_function_name(name, stmt.location)
+                    {
+                        self.checks.push(check);
+                    }
+                }
                 for expr in decorator_list {
                     self.visit_expr(expr);
                 }
@@ -261,6 +324,12 @@ where
                     }
                 }
 
+                if self.settings.select.contains(&CheckCode::E742) {
+                    if let Some(check) = checks::check_ambiguous_class_name(name, stmt.location) {
+                        self.checks.push(check);
+                    }
+                }
+
                 for expr in bases {
                     self.visit_expr(expr)
                 }
@@ -347,16 +416,36 @@ where
                             name,
                             Binding {
                                 kind: BindingKind::FutureImportation,
-                                used: Some(
+                                used: Some((
                                     self.scopes[*(self
                                         .scope_stack
                                         .last()
                                         .expect("No current scope found."))]
                                     .id,
-                                ),
+                                    stmt.location,
+                                )),
                                 location: stmt.location,
                             },
                         );
+
+                        if alias.node.name == "annotations" {
+                            self.annotations_future_enabled = true;
+                        }
+
+                        if self.settings.select.contains(&CheckCode::F407)
+                            && !ALL_FEATURE_NAMES.contains(&alias.node.name.deref())
+                        {
+                            self.checks.push(Check::new(
+                                CheckKind::FutureFeatureNotDefined(alias.node.name.to_string()),
+                                stmt.location,
+                            ));
+                        }
+
+                        if self.settings.select.contains(&CheckCode::F404) && !self.futures_allowed
+                        {
+                            self.checks
+                                .push(Check::new(CheckKind::LateFutureImport, stmt.location));
+                        }
                     } else if alias.node.name == "*" {
                         self.add_binding(
                             name,
@@ -368,14 +457,31 @@ where
                         );
 
                         if self.settings.select.contains(&CheckCode::F403) {
-                            self.checks
-                                .push(Check::new(CheckKind::ImportStarUsage, stmt.location));
+                            self.checks.push(Check::new(
+                                CheckKind::ImportStarUsage(
+                                    module.clone().unwrap_or_else(|| "module".to_string()),
+                                ),
+                                stmt.location,
+                            ));
+                        }
+
+                        if self.settings.select.contains(&CheckCode::F406) {
+                            let scope = &self.scopes
+                                [*(self.scope_stack.last().expect("No current scope found."))];
+                            if !matches!(scope.kind, ScopeKind::Module) {
+                                self.checks.push(Check::new(
+                                    CheckKind::ImportStarNotPermitted(
+                                        module.clone().unwrap_or_else(|| "module".to_string()),
+                                    ),
+                                    stmt.location,
+                                ));
+                            }
                         }
                     } else {
                         let binding = Binding {
                             kind: BindingKind::Importation(match module {
                                 None => name.clone(),
-                                Some(parent) => format!("{}.{}", parent, name.clone()),
+                                Some(parent) => format!("{}.{}", parent, name),
                             }),
                             used: None,
                             location: stmt.location,
@@ -484,6 +590,17 @@ where
         let prev_in_literal = self.in_literal;
         let prev_in_annotation = self.in_annotation;
 
+        // Important:
+        if self.in_annotation && self.annotations_future_enabled {
+            self.deferred_annotations.push((
+                expr,
+                self.scope_stack.clone(),
+                self.parent_stack.clone(),
+            ));
+            visitor::walk_expr(self, expr);
+            return;
+        }
+
         // Pre-visit.
         match &expr.node {
             ExprKind::Subscript { value, .. } => {
@@ -491,7 +608,7 @@ where
                     self.in_literal = true;
                 }
             }
-            ExprKind::Tuple { elts, ctx } => {
+            ExprKind::Tuple { elts, ctx } | ExprKind::List { elts, ctx } => {
                 if matches!(ctx, ExprContext::Store) {
                     let check_too_many_expressions =
                         self.settings.select.contains(&CheckCode::F621);
@@ -507,12 +624,19 @@ where
                     }
                 }
             }
-            ExprKind::Name { ctx, .. } => match ctx {
+            ExprKind::Name { id, ctx } => match ctx {
                 ExprContext::Load => self.handle_node_load(expr),
                 ExprContext::Store => {
+                    if self.settings.select.contains(&CheckCode::E741) {
+                        if let Some(check) =
+                            checks::check_ambiguous_variable_name(id, expr.location)
+                        {
+                            self.checks.push(check);
+                        }
+                    }
                     let parent =
                         self.parents[*(self.parent_stack.last().expect("No parent found."))];
-                    self.handle_node_store(expr, Some(parent));
+                    self.handle_node_store(expr, parent);
                 }
                 ExprContext::Del => self.handle_node_delete(expr),
             },
@@ -541,8 +665,7 @@ where
                     .settings
                     .select
                     .contains(CheckKind::YieldOutsideFunction.code())
-                    && matches!(scope.kind, ScopeKind::Class)
-                    || matches!(scope.kind, ScopeKind::Module)
+                    && matches!(scope.kind, ScopeKind::Class | ScopeKind::Module)
                 {
                     self.checks
                         .push(Check::new(CheckKind::YieldOutsideFunction, expr.location));
@@ -565,6 +688,28 @@ where
                 }
                 self.in_f_string = true;
             }
+            ExprKind::BinOp {
+                left,
+                op: Operator::RShift,
+                ..
+            } => {
+                if self.settings.select.contains(&CheckCode::F633) {
+                    if let ExprKind::Name { id, .. } = &left.node {
+                        if id == "print" {
+                            let scope = &self.scopes
+                                [*(self.scope_stack.last().expect("No current scope found."))];
+                            if let Some(Binding {
+                                kind: BindingKind::Builtin,
+                                ..
+                            }) = scope.values.get("print")
+                            {
+                                self.checks
+                                    .push(Check::new(CheckKind::InvalidPrintSyntax, left.location));
+                            }
+                        }
+                    }
+                }
+            }
             ExprKind::UnaryOp { op, operand } => {
                 let check_not_in = self.settings.select.contains(&CheckCode::E713);
                 let check_not_is = self.settings.select.contains(&CheckCode::E714);
@@ -593,12 +738,30 @@ where
                         check_true_false_comparisons,
                     ));
                 }
+
+                if self.settings.select.contains(&CheckCode::F632) {
+                    self.checks.extend(checks::check_is_literal(
+                        left,
+                        ops,
+                        comparators,
+                        expr.location,
+                    ));
+                }
+
+                if self.settings.select.contains(&CheckCode::E721) {
+                    self.checks.extend(checks::check_type_comparison(
+                        ops,
+                        comparators,
+                        expr.location,
+                    ));
+                }
             }
             ExprKind::Constant {
                 value: Constant::Str(value),
                 ..
             } if self.in_annotation && !self.in_literal => {
-                self.deferred_annotations.push((expr.location, value));
+                self.deferred_string_annotations
+                    .push((expr.location, value));
             }
             ExprKind::GeneratorExp { .. }
             | ExprKind::ListComp { .. }
@@ -659,7 +822,7 @@ where
                 } else if match_name_or_attr(func, "NamedTuple") {
                     self.visit_expr(func);
 
-                    // NamedTuple("a", [("a", int)])
+                    // Ex) NamedTuple("a", [("a", int)])
                     if args.len() > 1 {
                         match &args[1].node {
                             ExprKind::List { elts, .. } | ExprKind::Tuple { elts, .. } => {
@@ -683,7 +846,7 @@ where
                         }
                     }
 
-                    // NamedTuple("a", a=int)
+                    // Ex) NamedTuple("a", a=int)
                     for keyword in keywords {
                         let KeywordData { value, .. } = &keyword.node;
                         self.visit_annotation(value);
@@ -691,7 +854,7 @@ where
                 } else if match_name_or_attr(func, "TypedDict") {
                     self.visit_expr(func);
 
-                    // TypedDict("a", {"a": int})
+                    // Ex) TypedDict("a", {"a": int})
                     if args.len() > 1 {
                         if let ExprKind::Dict { keys, values } = &args[1].node {
                             for key in keys {
@@ -705,7 +868,7 @@ where
                         }
                     }
 
-                    // TypedDict("a", a=int)
+                    // Ex) TypedDict("a", a=int)
                     for keyword in keywords {
                         let KeywordData { value, .. } = &keyword.node;
                         self.visit_annotation(value);
@@ -744,13 +907,45 @@ where
 
     fn visit_excepthandler(&mut self, excepthandler: &'b Excepthandler) {
         match &excepthandler.node {
-            ExcepthandlerKind::ExceptHandler { name, .. } => match name {
-                Some(name) => {
-                    let scope =
-                        &self.scopes[*(self.scope_stack.last().expect("No current scope found."))];
-                    if scope.values.contains_key(name) {
+            ExcepthandlerKind::ExceptHandler { type_, name, .. } => {
+                if self.settings.select.contains(&CheckCode::E722) && type_.is_none() {
+                    self.checks.push(Check::new(
+                        CheckKind::DoNotUseBareExcept,
+                        excepthandler.location,
+                    ));
+                }
+                match name {
+                    Some(name) => {
+                        if self.settings.select.contains(&CheckCode::E741) {
+                            if let Some(check) =
+                                checks::check_ambiguous_variable_name(name, excepthandler.location)
+                            {
+                                self.checks.push(check);
+                            }
+                        }
+                        let scope = &self.scopes
+                            [*(self.scope_stack.last().expect("No current scope found."))];
+                        if scope.values.contains_key(name) {
+                            let parent = self.parents
+                                [*(self.parent_stack.last().expect("No parent found."))];
+                            self.handle_node_store(
+                                &Expr::new(
+                                    excepthandler.location,
+                                    ExprKind::Name {
+                                        id: name.to_string(),
+                                        ctx: ExprContext::Store,
+                                    },
+                                ),
+                                parent,
+                            );
+                            self.parents.push(parent);
+                        }
+
                         let parent =
                             self.parents[*(self.parent_stack.last().expect("No parent found."))];
+                        let scope = &self.scopes
+                            [*(self.scope_stack.last().expect("No current scope found."))];
+                        let definition = scope.values.get(name).cloned();
                         self.handle_node_store(
                             &Expr::new(
                                 excepthandler.location,
@@ -759,48 +954,32 @@ where
                                     ctx: ExprContext::Store,
                                 },
                             ),
-                            Some(parent),
+                            parent,
                         );
                         self.parents.push(parent);
-                    }
 
-                    let parent =
-                        self.parents[*(self.parent_stack.last().expect("No parent found."))];
-                    let scope =
-                        &self.scopes[*(self.scope_stack.last().expect("No current scope found."))];
-                    let definition = scope.values.get(name).cloned();
-                    self.handle_node_store(
-                        &Expr::new(
-                            excepthandler.location,
-                            ExprKind::Name {
-                                id: name.to_string(),
-                                ctx: ExprContext::Store,
-                            },
-                        ),
-                        Some(parent),
-                    );
-                    self.parents.push(parent);
+                        walk_excepthandler(self, excepthandler);
 
-                    walk_excepthandler(self, excepthandler);
+                        let scope = &mut self.scopes
+                            [*(self.scope_stack.last().expect("No current scope found."))];
+                        if let Some(binding) = &scope.values.remove(name) {
+                            if self.settings.select.contains(&CheckCode::F841)
+                                && binding.used.is_none()
+                            {
+                                self.checks.push(Check::new(
+                                    CheckKind::UnusedVariable(name.to_string()),
+                                    excepthandler.location,
+                                ));
+                            }
+                        }
 
-                    let scope = &mut self.scopes
-                        [*(self.scope_stack.last().expect("No current scope found."))];
-                    if let Some(binding) = &scope.values.remove(name) {
-                        if self.settings.select.contains(&CheckCode::F841) && binding.used.is_none()
-                        {
-                            self.checks.push(Check::new(
-                                CheckKind::UnusedVariable(name.to_string()),
-                                excepthandler.location,
-                            ));
+                        if let Some(binding) = definition {
+                            scope.values.insert(name.to_string(), binding);
                         }
                     }
-
-                    if let Some(binding) = definition {
-                        scope.values.insert(name.to_string(), binding);
-                    }
+                    None => walk_excepthandler(self, excepthandler),
                 }
-                None => walk_excepthandler(self, excepthandler),
-            },
+            }
         }
     }
 
@@ -838,6 +1017,13 @@ where
                 location: arg.location,
             },
         );
+
+        if self.settings.select.contains(&CheckCode::E741) {
+            if let Some(check) = checks::check_ambiguous_variable_name(&arg.node.arg, arg.location)
+            {
+                self.checks.push(check);
+            }
+        }
     }
 }
 
@@ -871,7 +1057,7 @@ impl<'a> Checker<'a> {
 
         for builtin in BUILTINS {
             scope.values.insert(
-                builtin.to_string(),
+                (*builtin).to_string(),
                 Binding {
                     kind: BindingKind::Builtin,
                     location: Default::default(),
@@ -881,7 +1067,7 @@ impl<'a> Checker<'a> {
         }
         for builtin in MAGIC_GLOBALS {
             scope.values.insert(
-                builtin.to_string(),
+                (*builtin).to_string(),
                 Binding {
                     kind: BindingKind::Builtin,
                     location: Default::default(),
@@ -923,7 +1109,7 @@ impl<'a> Checker<'a> {
                     }
                 }
                 if let Some(binding) = scope.values.get_mut(id) {
-                    binding.used = Some(scope_id);
+                    binding.used = Some((scope_id, expr.location));
                     return;
                 }
 
@@ -932,6 +1118,10 @@ impl<'a> Checker<'a> {
             }
 
             if self.settings.select.contains(&CheckCode::F821) {
+                // Allow __path__.
+                if self.path.ends_with("__init__.py") && id == "__path__" {
+                    return;
+                }
                 self.checks.push(Check::new(
                     CheckKind::UndefinedName(id.clone()),
                     expr.location,
@@ -940,7 +1130,7 @@ impl<'a> Checker<'a> {
         }
     }
 
-    fn handle_node_store(&mut self, expr: &Expr, parent: Option<&Stmt>) {
+    fn handle_node_store(&mut self, expr: &Expr, parent: &Stmt) {
         if let ExprKind::Name { id, .. } = &expr.node {
             let current =
                 &self.scopes[*(self.scope_stack.last().expect("No current scope found."))];
@@ -950,55 +1140,78 @@ impl<'a> Checker<'a> {
                 && !current.values.contains_key(id)
             {
                 for scope in self.scopes.iter().rev().skip(1) {
-                    if matches!(scope.kind, ScopeKind::Function)
-                        || matches!(scope.kind, ScopeKind::Module)
-                    {
-                        let used = scope
-                            .values
-                            .get(id)
-                            .map(|binding| binding.used)
-                            .unwrap_or_default();
-                        if let Some(scope_id) = used {
-                            if scope_id == current.id {
-                                self.checks.push(Check::new(
-                                    CheckKind::UndefinedLocal(id.clone()),
-                                    expr.location,
-                                ));
+                    if matches!(scope.kind, ScopeKind::Function | ScopeKind::Module) {
+                        if let Some(binding) = scope.values.get(id) {
+                            if let Some((scope_id, location)) = binding.used {
+                                if scope_id == current.id {
+                                    self.checks.push(Check::new(
+                                        CheckKind::UndefinedLocal(id.clone()),
+                                        location,
+                                    ));
+                                }
                             }
                         }
                     }
                 }
             }
 
-            // TODO(charlie): Handle alternate binding types (like `Annotation`).
-            if id == "__all__"
-                && matches!(current.kind, ScopeKind::Module)
-                && parent
-                    .map(|stmt| {
-                        matches!(stmt.node, StmtKind::Assign { .. })
-                            || matches!(stmt.node, StmtKind::AugAssign { .. })
-                            || matches!(stmt.node, StmtKind::AnnAssign { .. })
-                    })
-                    .unwrap_or_default()
+            if matches!(parent.node, StmtKind::AnnAssign { value: None, .. }) {
+                self.add_binding(
+                    id.to_string(),
+                    Binding {
+                        kind: BindingKind::Annotation,
+                        used: None,
+                        location: expr.location,
+                    },
+                );
+                return;
+            }
+
+            // TODO(charlie): Include comprehensions here.
+            if matches!(
+                parent.node,
+                StmtKind::For { .. } | StmtKind::AsyncFor { .. }
+            ) || operations::is_unpacking_assignment(parent)
             {
                 self.add_binding(
                     id.to_string(),
                     Binding {
-                        kind: BindingKind::Export(extract_all_names(parent.unwrap(), current)),
+                        kind: BindingKind::Binding,
                         used: None,
                         location: expr.location,
                     },
                 );
-            } else {
+                return;
+            }
+
+            if id == "__all__"
+                && matches!(current.kind, ScopeKind::Module)
+                && matches!(
+                    parent.node,
+                    StmtKind::Assign { .. }
+                        | StmtKind::AugAssign { .. }
+                        | StmtKind::AnnAssign { .. }
+                )
+            {
                 self.add_binding(
                     id.to_string(),
                     Binding {
-                        kind: BindingKind::Assignment,
+                        kind: BindingKind::Export(extract_all_names(parent, current)),
                         used: None,
                         location: expr.location,
                     },
                 );
+                return;
             }
+
+            self.add_binding(
+                id.to_string(),
+                Binding {
+                    kind: BindingKind::Assignment,
+                    used: None,
+                    location: expr.location,
+                },
+            );
         }
     }
 
@@ -1021,15 +1234,27 @@ impl<'a> Checker<'a> {
         }
     }
 
-    fn check_deferred_annotations<'b>(&mut self, path: &str, allocator: &'b mut Vec<Expr>)
+    fn check_deferred_annotations(&mut self) {
+        while let Some((expr, scopes, parents)) = self.deferred_annotations.pop() {
+            self.parent_stack = parents;
+            self.scope_stack = scopes;
+            self.visit_expr(expr);
+        }
+    }
+
+    fn check_deferred_string_annotations<'b>(&mut self, allocator: &'b mut Vec<Expr>)
     where
         'b: 'a,
     {
-        while !self.deferred_annotations.is_empty() {
-            let (location, expression) = self.deferred_annotations.pop().unwrap();
-            if let Ok(mut expr) = parser::parse_expression(expression, path) {
+        while let Some((location, expression)) = self.deferred_string_annotations.pop() {
+            if let Ok(mut expr) = parser::parse_expression(expression, "<filename>") {
                 relocate_expr(&mut expr, location);
                 allocator.push(expr);
+            } else if self.settings.select.contains(&CheckCode::F722) {
+                self.checks.push(Check::new(
+                    CheckKind::ForwardAnnotationSyntaxError(expression.to_string()),
+                    location,
+                ));
             }
         }
         for expr in allocator {
@@ -1038,9 +1263,7 @@ impl<'a> Checker<'a> {
     }
 
     fn check_deferred_functions(&mut self) {
-        while !self.deferred_functions.is_empty() {
-            let (stmt, scopes, parents) = self.deferred_functions.pop().unwrap();
-
+        while let Some((stmt, scopes, parents)) = self.deferred_functions.pop() {
             self.parent_stack = parents;
             self.scope_stack = scopes;
             self.push_scope(Scope::new(ScopeKind::Function));
@@ -1064,9 +1287,7 @@ impl<'a> Checker<'a> {
     }
 
     fn check_deferred_lambdas(&mut self) {
-        while !self.deferred_lambdas.is_empty() {
-            let (expr, scopes, parents) = self.deferred_lambdas.pop().unwrap();
-
+        while let Some((expr, scopes, parents)) = self.deferred_lambdas.pop() {
             self.parent_stack = parents;
             self.scope_stack = scopes;
             self.push_scope(Scope::new(ScopeKind::Function));
@@ -1084,8 +1305,7 @@ impl<'a> Checker<'a> {
     }
 
     fn check_deferred_assignments(&mut self) {
-        while !self.deferred_assignments.is_empty() {
-            let index = self.deferred_assignments.pop().unwrap();
+        while let Some(index) = self.deferred_assignments.pop() {
             if self.settings.select.contains(&CheckCode::F841) {
                 self.checks
                     .extend(checks::check_unused_variables(&self.scopes[index]));
@@ -1100,7 +1320,7 @@ impl<'a> Checker<'a> {
             return;
         }
 
-        for index in self.dead_scopes.clone() {
+        for index in self.dead_scopes.iter().copied() {
             let scope = &self.scopes[index];
 
             let all_binding = scope.values.get("__all__");
@@ -1110,7 +1330,7 @@ impl<'a> Checker<'a> {
             });
 
             if self.settings.select.contains(&CheckCode::F822)
-                && !Path::new(self.path).ends_with("__init__.py")
+                && !self.path.ends_with("__init__.py")
             {
                 if let Some(binding) = all_binding {
                     if let Some(names) = all_names {
@@ -1156,7 +1376,7 @@ pub fn check_ast(
     content: &str,
     settings: &Settings,
     autofix: &fixer::Mode,
-    path: &str,
+    path: &Path,
 ) -> Vec<Check> {
     let mut checker = Checker::new(settings, autofix, path, content);
     checker.push_scope(Scope::new(ScopeKind::Module));
@@ -1171,8 +1391,9 @@ pub fn check_ast(
     checker.check_deferred_functions();
     checker.check_deferred_lambdas();
     checker.check_deferred_assignments();
+    checker.check_deferred_annotations();
     let mut allocator = vec![];
-    checker.check_deferred_annotations(path, &mut allocator);
+    checker.check_deferred_string_annotations(&mut allocator);
 
     // Reset the scope to module-level, and check all consumed scopes.
     checker.scope_stack = vec![GLOBAL_SCOPE_INDEX];
