@@ -14,20 +14,34 @@ pub enum CheckCode {
     E712,
     E713,
     E714,
+    E721,
+    E722,
     E731,
+    E741,
+    E742,
+    E743,
     E902,
+    E999,
     F401,
     F403,
+    F404,
+    F406,
+    F407,
     F541,
     F601,
     F602,
     F621,
     F622,
     F631,
+    F632,
+    F633,
     F634,
+    F701,
+    F702,
     F704,
     F706,
     F707,
+    F722,
     F821,
     F822,
     F823,
@@ -49,20 +63,34 @@ impl FromStr for CheckCode {
             "E712" => Ok(CheckCode::E712),
             "E713" => Ok(CheckCode::E713),
             "E714" => Ok(CheckCode::E714),
+            "E721" => Ok(CheckCode::E721),
+            "E722" => Ok(CheckCode::E722),
             "E731" => Ok(CheckCode::E731),
+            "E741" => Ok(CheckCode::E741),
+            "E742" => Ok(CheckCode::E742),
+            "E743" => Ok(CheckCode::E743),
             "E902" => Ok(CheckCode::E902),
+            "E999" => Ok(CheckCode::E999),
             "F401" => Ok(CheckCode::F401),
             "F403" => Ok(CheckCode::F403),
+            "F404" => Ok(CheckCode::F404),
+            "F406" => Ok(CheckCode::F406),
+            "F407" => Ok(CheckCode::F407),
             "F541" => Ok(CheckCode::F541),
             "F601" => Ok(CheckCode::F601),
             "F602" => Ok(CheckCode::F602),
             "F621" => Ok(CheckCode::F621),
             "F622" => Ok(CheckCode::F622),
             "F631" => Ok(CheckCode::F631),
+            "F632" => Ok(CheckCode::F632),
+            "F633" => Ok(CheckCode::F633),
             "F634" => Ok(CheckCode::F634),
+            "F701" => Ok(CheckCode::F701),
+            "F702" => Ok(CheckCode::F702),
             "F704" => Ok(CheckCode::F704),
             "F706" => Ok(CheckCode::F706),
             "F707" => Ok(CheckCode::F707),
+            "F722" => Ok(CheckCode::F722),
             "F821" => Ok(CheckCode::F821),
             "F822" => Ok(CheckCode::F822),
             "F823" => Ok(CheckCode::F823),
@@ -85,20 +113,34 @@ impl CheckCode {
             CheckCode::E712 => "E712",
             CheckCode::E713 => "E713",
             CheckCode::E714 => "E714",
+            CheckCode::E721 => "E721",
+            CheckCode::E722 => "E722",
             CheckCode::E731 => "E731",
+            CheckCode::E741 => "E741",
+            CheckCode::E742 => "E742",
+            CheckCode::E743 => "E743",
             CheckCode::E902 => "E902",
+            CheckCode::E999 => "E999",
             CheckCode::F401 => "F401",
             CheckCode::F403 => "F403",
+            CheckCode::F404 => "F404",
+            CheckCode::F406 => "F406",
+            CheckCode::F407 => "F407",
             CheckCode::F541 => "F541",
             CheckCode::F601 => "F601",
             CheckCode::F602 => "F602",
             CheckCode::F621 => "F621",
             CheckCode::F622 => "F622",
             CheckCode::F631 => "F631",
+            CheckCode::F632 => "F632",
+            CheckCode::F633 => "F633",
             CheckCode::F634 => "F634",
+            CheckCode::F701 => "F701",
+            CheckCode::F702 => "F702",
             CheckCode::F704 => "F704",
             CheckCode::F706 => "F706",
             CheckCode::F707 => "F707",
+            CheckCode::F722 => "F722",
             CheckCode::F821 => "F821",
             CheckCode::F822 => "F822",
             CheckCode::F823 => "F823",
@@ -110,37 +152,12 @@ impl CheckCode {
         }
     }
 
-    /// The source for the check (either the AST, or the physical lines).
+    /// The source for the check (either the AST, the filesystem, or the physical lines).
     pub fn lint_source(&self) -> &'static LintSource {
         match self {
-            CheckCode::E402 => &LintSource::AST,
             CheckCode::E501 => &LintSource::Lines,
-            CheckCode::E711 => &LintSource::AST,
-            CheckCode::E712 => &LintSource::AST,
-            CheckCode::E713 => &LintSource::AST,
-            CheckCode::E714 => &LintSource::AST,
-            CheckCode::E731 => &LintSource::AST,
-            CheckCode::E902 => &LintSource::FileSystem,
-            CheckCode::F401 => &LintSource::AST,
-            CheckCode::F403 => &LintSource::AST,
-            CheckCode::F541 => &LintSource::AST,
-            CheckCode::F601 => &LintSource::AST,
-            CheckCode::F602 => &LintSource::AST,
-            CheckCode::F621 => &LintSource::AST,
-            CheckCode::F622 => &LintSource::AST,
-            CheckCode::F631 => &LintSource::AST,
-            CheckCode::F634 => &LintSource::AST,
-            CheckCode::F704 => &LintSource::AST,
-            CheckCode::F706 => &LintSource::AST,
-            CheckCode::F707 => &LintSource::AST,
-            CheckCode::F821 => &LintSource::AST,
-            CheckCode::F822 => &LintSource::AST,
-            CheckCode::F823 => &LintSource::AST,
-            CheckCode::F831 => &LintSource::AST,
-            CheckCode::F841 => &LintSource::AST,
-            CheckCode::F901 => &LintSource::AST,
-            CheckCode::R001 => &LintSource::AST,
-            CheckCode::R002 => &LintSource::AST,
+            CheckCode::E902 | CheckCode::E999 => &LintSource::FileSystem,
+            _ => &LintSource::AST,
         }
     }
 }
@@ -160,15 +177,27 @@ pub enum RejectedCmpop {
 
 #[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
 pub enum CheckKind {
+    AmbiguousClassName(String),
+    AmbiguousFunctionName(String),
+    AmbiguousVariableName(String),
     AssertTuple,
+    BreakOutsideLoop,
+    ContinueOutsideLoop,
     DefaultExceptNotLast,
     DoNotAssignLambda,
+    DoNotUseBareExcept,
     DuplicateArgumentName,
+    ForwardAnnotationSyntaxError(String),
     FStringMissingPlaceholders,
+    FutureFeatureNotDefined(String),
     IOError(String),
     IfTuple,
-    ImportStarUsage,
-    LineTooLong,
+    ImportStarNotPermitted(String),
+    ImportStarUsage(String),
+    InvalidPrintSyntax,
+    IsLiteral,
+    LateFutureImport,
+    LineTooLong(usize, usize),
     ModuleImportNotAtTopOfFile,
     MultiValueRepeatedKeyLiteral,
     MultiValueRepeatedKeyVariable(String),
@@ -178,9 +207,11 @@ pub enum CheckKind {
     NotIsTest,
     RaiseNotImplemented,
     ReturnOutsideFunction,
+    SyntaxError(String),
     TooManyExpressionsInStarredAssignment,
     TrueFalseComparison(bool, RejectedCmpop),
     TwoStarredExpressions,
+    TypeComparison,
     UndefinedExport(String),
     UndefinedLocal(String),
     UndefinedName(String),
@@ -194,15 +225,27 @@ impl CheckKind {
     /// The name of the check.
     pub fn name(&self) -> &'static str {
         match self {
+            CheckKind::AmbiguousClassName(_) => "AmbiguousClassName",
+            CheckKind::AmbiguousFunctionName(_) => "AmbiguousFunctionName",
+            CheckKind::AmbiguousVariableName(_) => "AmbiguousVariableName",
             CheckKind::AssertTuple => "AssertTuple",
+            CheckKind::BreakOutsideLoop => "BreakOutsideLoop",
+            CheckKind::ContinueOutsideLoop => "ContinueOutsideLoop",
             CheckKind::DefaultExceptNotLast => "DefaultExceptNotLast",
+            CheckKind::DoNotAssignLambda => "DoNotAssignLambda",
+            CheckKind::DoNotUseBareExcept => "DoNotUseBareExcept",
             CheckKind::DuplicateArgumentName => "DuplicateArgumentName",
             CheckKind::FStringMissingPlaceholders => "FStringMissingPlaceholders",
+            CheckKind::ForwardAnnotationSyntaxError(_) => "ForwardAnnotationSyntaxError",
+            CheckKind::FutureFeatureNotDefined(_) => "FutureFeatureNotDefined",
             CheckKind::IOError(_) => "IOError",
             CheckKind::IfTuple => "IfTuple",
-            CheckKind::ImportStarUsage => "ImportStarUsage",
-            CheckKind::LineTooLong => "LineTooLong",
-            CheckKind::DoNotAssignLambda => "DoNotAssignLambda",
+            CheckKind::ImportStarNotPermitted(_) => "ImportStarNotPermitted",
+            CheckKind::ImportStarUsage(_) => "ImportStarUsage",
+            CheckKind::InvalidPrintSyntax => "InvalidPrintSyntax",
+            CheckKind::IsLiteral => "IsLiteral",
+            CheckKind::LateFutureImport => "LateFutureImport",
+            CheckKind::LineTooLong(_, _) => "LineTooLong",
             CheckKind::ModuleImportNotAtTopOfFile => "ModuleImportNotAtTopOfFile",
             CheckKind::MultiValueRepeatedKeyLiteral => "MultiValueRepeatedKeyLiteral",
             CheckKind::MultiValueRepeatedKeyVariable(_) => "MultiValueRepeatedKeyVariable",
@@ -212,11 +255,13 @@ impl CheckKind {
             CheckKind::NotIsTest => "NotIsTest",
             CheckKind::RaiseNotImplemented => "RaiseNotImplemented",
             CheckKind::ReturnOutsideFunction => "ReturnOutsideFunction",
+            CheckKind::SyntaxError(_) => "SyntaxError",
             CheckKind::TooManyExpressionsInStarredAssignment => {
                 "TooManyExpressionsInStarredAssignment"
             }
             CheckKind::TrueFalseComparison(_, _) => "TrueFalseComparison",
             CheckKind::TwoStarredExpressions => "TwoStarredExpressions",
+            CheckKind::TypeComparison => "TypeComparison",
             CheckKind::UndefinedExport(_) => "UndefinedExport",
             CheckKind::UndefinedLocal(_) => "UndefinedLocal",
             CheckKind::UndefinedName(_) => "UndefinedName",
@@ -230,15 +275,27 @@ impl CheckKind {
     /// A four-letter shorthand code for the check.
     pub fn code(&self) -> &'static CheckCode {
         match self {
+            CheckKind::AmbiguousClassName(_) => &CheckCode::E742,
+            CheckKind::AmbiguousFunctionName(_) => &CheckCode::E743,
+            CheckKind::AmbiguousVariableName(_) => &CheckCode::E741,
             CheckKind::AssertTuple => &CheckCode::F631,
+            CheckKind::BreakOutsideLoop => &CheckCode::F701,
+            CheckKind::ContinueOutsideLoop => &CheckCode::F702,
             CheckKind::DefaultExceptNotLast => &CheckCode::F707,
+            CheckKind::DoNotAssignLambda => &CheckCode::E731,
+            CheckKind::DoNotUseBareExcept => &CheckCode::E722,
             CheckKind::DuplicateArgumentName => &CheckCode::F831,
             CheckKind::FStringMissingPlaceholders => &CheckCode::F541,
+            CheckKind::ForwardAnnotationSyntaxError(_) => &CheckCode::F722,
+            CheckKind::FutureFeatureNotDefined(_) => &CheckCode::F407,
             CheckKind::IOError(_) => &CheckCode::E902,
             CheckKind::IfTuple => &CheckCode::F634,
-            CheckKind::ImportStarUsage => &CheckCode::F403,
-            CheckKind::LineTooLong => &CheckCode::E501,
-            CheckKind::DoNotAssignLambda => &CheckCode::E731,
+            CheckKind::ImportStarNotPermitted(_) => &CheckCode::F406,
+            CheckKind::ImportStarUsage(_) => &CheckCode::F403,
+            CheckKind::InvalidPrintSyntax => &CheckCode::F633,
+            CheckKind::IsLiteral => &CheckCode::F632,
+            CheckKind::LateFutureImport => &CheckCode::F404,
+            CheckKind::LineTooLong(_, _) => &CheckCode::E501,
             CheckKind::ModuleImportNotAtTopOfFile => &CheckCode::E402,
             CheckKind::MultiValueRepeatedKeyLiteral => &CheckCode::F601,
             CheckKind::MultiValueRepeatedKeyVariable(_) => &CheckCode::F602,
@@ -248,9 +305,11 @@ impl CheckKind {
             CheckKind::NotIsTest => &CheckCode::E714,
             CheckKind::RaiseNotImplemented => &CheckCode::F901,
             CheckKind::ReturnOutsideFunction => &CheckCode::F706,
+            CheckKind::SyntaxError(_) => &CheckCode::E999,
             CheckKind::TooManyExpressionsInStarredAssignment => &CheckCode::F621,
             CheckKind::TrueFalseComparison(_, _) => &CheckCode::E712,
             CheckKind::TwoStarredExpressions => &CheckCode::F622,
+            CheckKind::TypeComparison => &CheckCode::E721,
             CheckKind::UndefinedExport(_) => &CheckCode::F822,
             CheckKind::UndefinedLocal(_) => &CheckCode::F823,
             CheckKind::UndefinedName(_) => &CheckCode::F821,
@@ -264,26 +323,56 @@ impl CheckKind {
     /// The body text for the check.
     pub fn body(&self) -> String {
         match self {
+            CheckKind::AmbiguousClassName(name) => {
+                format!("ambiguous class name '{}'", name)
+            }
+            CheckKind::AmbiguousFunctionName(name) => {
+                format!("ambiguous function name '{}'", name)
+            }
+            CheckKind::AmbiguousVariableName(name) => {
+                format!("ambiguous variable name '{}'", name)
+            }
             CheckKind::AssertTuple => {
                 "Assert test is a non-empty tuple, which is always `True`".to_string()
             }
+            CheckKind::BreakOutsideLoop => "`break` outside loop".to_string(),
+            CheckKind::ContinueOutsideLoop => "`continue` not properly in loop".to_string(),
             CheckKind::DefaultExceptNotLast => {
                 "an `except:` block as not the last exception handler".to_string()
             }
+            CheckKind::DoNotAssignLambda => {
+                "Do not assign a lambda expression, use a def".to_string()
+            }
+            CheckKind::DoNotUseBareExcept => "Do not use bare `except`".to_string(),
             CheckKind::DuplicateArgumentName => {
                 "Duplicate argument name in function definition".to_string()
             }
+            CheckKind::ForwardAnnotationSyntaxError(body) => {
+                format!("syntax error in forward annotation '{body}'")
+            }
             CheckKind::FStringMissingPlaceholders => {
                 "f-string without any placeholders".to_string()
             }
+            CheckKind::FutureFeatureNotDefined(name) => {
+                format!("future feature '{name}' is not defined")
+            }
             CheckKind::IOError(name) => {
                 format!("No such file or directory: `{name}`")
             }
             CheckKind::IfTuple => "If test is a tuple, which is always `True`".to_string(),
-            CheckKind::ImportStarUsage => "Unable to detect undefined names".to_string(),
-            CheckKind::LineTooLong => "Line too long".to_string(),
-            CheckKind::DoNotAssignLambda => {
-                "Do not assign a lambda expression, use a def".to_string()
+            CheckKind::InvalidPrintSyntax => "use of >> is invalid with print function".to_string(),
+            CheckKind::ImportStarNotPermitted(name) => {
+                format!("`from {name} import *` only allowed at module level")
+            }
+            CheckKind::ImportStarUsage(name) => {
+                format!("`from {name} import *` used; unable to detect undefined names")
+            }
+            CheckKind::IsLiteral => "use ==/!= to compare constant literals".to_string(),
+            CheckKind::LateFutureImport => {
+                "from __future__ imports must occur at the beginning of the file".to_string()
+            }
+            CheckKind::LineTooLong(length, limit) => {
+                format!("Line too long ({length} > {limit} characters)")
             }
             CheckKind::ModuleImportNotAtTopOfFile => {
                 "Module level import not at top of file".to_string()
@@ -311,6 +400,7 @@ impl CheckKind {
             CheckKind::ReturnOutsideFunction => {
                 "a `return` statement outside of a function/method".to_string()
             }
+            CheckKind::SyntaxError(message) => format!("SyntaxError: {message}"),
             CheckKind::TooManyExpressionsInStarredAssignment => {
                 "too many expressions in star-unpacking assignment".to_string()
             }
@@ -333,15 +423,16 @@ impl CheckKind {
                 },
             },
             CheckKind::TwoStarredExpressions => "two starred expressions in assignment".to_string(),
+            CheckKind::TypeComparison => "do not compare types, use `isinstance()`".to_string(),
             CheckKind::UndefinedExport(name) => {
                 format!("Undefined name `{name}` in `__all__`")
             }
-            CheckKind::UndefinedName(name) => {
-                format!("Undefined name `{name}`")
-            }
             CheckKind::UndefinedLocal(name) => {
                 format!("Local variable `{name}` referenced before assignment")
             }
+            CheckKind::UndefinedName(name) => {
+                format!("Undefined name `{name}`")
+            }
             CheckKind::UnusedImport(name) => format!("`{name}` imported but unused"),
             CheckKind::UnusedVariable(name) => {
                 format!("Local variable `{name}` is assigned to but never used")
@@ -357,36 +448,10 @@ impl CheckKind {
 
     /// Whether the check kind is (potentially) fixable.
     pub fn fixable(&self) -> bool {
-        match self {
-            CheckKind::AssertTuple => false,
-            CheckKind::DefaultExceptNotLast => false,
-            CheckKind::DuplicateArgumentName => false,
-            CheckKind::FStringMissingPlaceholders => false,
-            CheckKind::IOError(_) => false,
-            CheckKind::IfTuple => false,
-            CheckKind::ImportStarUsage => false,
-            CheckKind::DoNotAssignLambda => false,
-            CheckKind::LineTooLong => false,
-            CheckKind::ModuleImportNotAtTopOfFile => false,
-            CheckKind::MultiValueRepeatedKeyLiteral => false,
-            CheckKind::MultiValueRepeatedKeyVariable(_) => false,
-            CheckKind::NoAssertEquals => true,
-            CheckKind::NotInTest => false,
-            CheckKind::NotIsTest => false,
-            CheckKind::NoneComparison(_) => false,
-            CheckKind::RaiseNotImplemented => false,
-            CheckKind::ReturnOutsideFunction => false,
-            CheckKind::TooManyExpressionsInStarredAssignment => false,
-            CheckKind::TrueFalseComparison(_, _) => false,
-            CheckKind::TwoStarredExpressions => false,
-            CheckKind::UndefinedExport(_) => false,
-            CheckKind::UndefinedLocal(_) => false,
-            CheckKind::UndefinedName(_) => false,
-            CheckKind::UnusedImport(_) => false,
-            CheckKind::UnusedVariable(_) => false,
-            CheckKind::UselessObjectInheritance(_) => true,
-            CheckKind::YieldOutsideFunction => false,
-        }
+        matches!(
+            self,
+            CheckKind::NoAssertEquals | CheckKind::UselessObjectInheritance(_)
+        )
     }
 }
 
