@@ -2,7 +2,7 @@ extern crate core;
 
 mod ast;
 mod autofix;
-mod cache;
+pub mod cache;
 pub mod check_ast;
 mod check_lines;
 pub mod checks;
@@ -10,6 +10,7 @@ pub mod fs;
 pub mod linter;
 pub mod logging;
 pub mod message;
+pub mod printer;
 mod pyproject;
 mod python;
 pub mod settings;
