@@ -1,4 +1,4 @@
-use std::path::{Path, PathBuf};
+use std::path::PathBuf;
 use std::process::ExitCode;
 use std::sync::mpsc::channel;
 use std::time::Instant;
@@ -12,12 +12,14 @@ use notify::{raw_watcher, RecursiveMode, Watcher};
 use rayon::prelude::*;
 use walkdir::DirEntry;
 
+use ::ruff::cache;
 use ::ruff::checks::CheckCode;
 use ::ruff::checks::CheckKind;
 use ::ruff::fs::iter_python_files;
 use ::ruff::linter::lint_path;
 use ::ruff::logging::set_up_logging;
 use ::ruff::message::Message;
+use ::ruff::printer::{Printer, SerializationFormat};
 use ::ruff::settings::Settings;
 use ::ruff::tell_user;
 
@@ -54,9 +56,15 @@ struct Cli {
     /// List of error codes to ignore.
     #[clap(long, multiple = true)]
     ignore: Vec<CheckCode>,
-    /// List of file and/or directory patterns to exclude from checks.
+    /// List of paths, used to exclude files and/or directories from checks.
     #[clap(long, multiple = true)]
     exclude: Vec<Pattern>,
+    /// Like --exclude, but adds additional files and directories on top of the excluded ones.
+    #[clap(long, multiple = true)]
+    extend_exclude: Vec<Pattern>,
+    /// Output serialization format for error messages.
+    #[clap(long, arg_enum, default_value_t=SerializationFormat::Text)]
+    format: SerializationFormat,
 }
 
 #[cfg(feature = "update-informer")]
@@ -93,7 +101,7 @@ fn run_once(
     let start = Instant::now();
     let paths: Vec<DirEntry> = files
         .iter()
-        .flat_map(|path| iter_python_files(path, &settings.exclude))
+        .flat_map(|path| iter_python_files(path, &settings.exclude, &settings.extend_exclude))
         .collect();
     let duration = start.elapsed();
     debug!("Identified files to lint in: {:?}", duration);
@@ -103,8 +111,17 @@ fn run_once(
         .par_iter()
         .map(|entry| {
             lint_path(entry.path(), settings, &cache.into(), &autofix.into()).unwrap_or_else(|e| {
-                error!("Failed to check {}: {e:?}", entry.path().to_string_lossy());
-                vec![]
+                if settings.select.contains(&CheckCode::E999) {
+                    vec![Message {
+                        kind: CheckKind::SyntaxError(e.to_string()),
+                        fixed: false,
+                        location: Default::default(),
+                        filename: entry.path().to_string_lossy().to_string(),
+                    }]
+                } else {
+                    error!("Failed to check {}: {e:?}", entry.path().to_string_lossy());
+                    vec![]
+                }
             })
         })
         .flatten()
@@ -130,62 +147,15 @@ fn run_once(
     Ok(messages)
 }
 
-fn report_once(messages: &[Message]) -> Result<()> {
-    let (fixed, outstanding): (Vec<&Message>, Vec<&Message>) =
-        messages.iter().partition(|message| message.fixed);
-    let num_fixable = outstanding
-        .iter()
-        .filter(|message| message.kind.fixable())
-        .count();
-
-    if !outstanding.is_empty() {
-        for message in &outstanding {
-            println!("{}", message);
-        }
-        println!();
-    }
-
-    if !fixed.is_empty() {
-        println!(
-            "Found {} error(s) ({} fixed).",
-            outstanding.len(),
-            fixed.len()
-        );
-    } else {
-        println!("Found {} error(s).", outstanding.len());
-    }
-
-    if num_fixable > 0 {
-        println!("{num_fixable} potentially fixable with the --fix option.");
-    }
-
-    Ok(())
-}
-
-fn report_continuously(messages: &[Message]) -> Result<()> {
-    tell_user!(
-        "Found {} error(s). Watching for file changes.",
-        messages.len(),
-    );
-
-    if !messages.is_empty() {
-        println!();
-        for message in messages {
-            println!("{}", message);
-        }
-    }
-
-    Ok(())
-}
-
 fn inner_main() -> Result<ExitCode> {
     let cli = Cli::parse();
 
     set_up_logging(cli.verbose)?;
 
-    // TODO(charlie): Can we avoid this cast?
-    let paths: Vec<&Path> = cli.files.iter().map(PathBuf::as_path).collect();
-    let mut settings = Settings::from_paths(paths)?;
+    let mut settings = Settings::from_paths(&cli.files);
+
+    let mut printer = Printer::new(cli.format);
+
     if !cli.select.is_empty() {
         settings.select(cli.select);
     }
@@ -193,21 +163,30 @@ fn inner_main() -> Result<ExitCode> {
         settings.ignore(&cli.ignore);
     }
     if !cli.exclude.is_empty() {
-        settings.exclude(cli.exclude);
+        settings.exclude = cli.exclude;
     }
+    if !cli.extend_exclude.is_empty() {
+        settings.extend_exclude = cli.extend_exclude;
+    }
+
+    cache::init()?;
 
     if cli.watch {
         if cli.fix {
-            println!("Warning: --fix is not enabled in watch mode.")
+            println!("Warning: --fix is not enabled in watch mode.");
+        }
+
+        if cli.format != SerializationFormat::Text {
+            println!("Warning: --format 'text' is used in watch mode.");
         }
 
         // Perform an initial run instantly.
-        clearscreen::clear()?;
+        printer.clear_screen()?;
         tell_user!("Starting linter in watch mode...\n");
 
         let messages = run_once(&cli.files, &settings, !cli.no_cache, false)?;
         if !cli.quiet {
-            report_continuously(&messages)?;
+            printer.write_continuously(&messages)?;
         }
 
         // Configure the file watcher.
@@ -222,12 +201,12 @@ fn inner_main() -> Result<ExitCode> {
                 Ok(e) => {
                     if let Some(path) = e.path {
                         if path.to_string_lossy().ends_with(".py") {
-                            clearscreen::clear()?;
+                            printer.clear_screen()?;
                             tell_user!("File change detected...\n");
 
                             let messages = run_once(&cli.files, &settings, !cli.no_cache, false)?;
                             if !cli.quiet {
-                                report_continuously(&messages)?;
+                                printer.write_continuously(&messages)?;
                             }
                         }
                     }
@@ -238,7 +217,7 @@ fn inner_main() -> Result<ExitCode> {
     } else {
         let messages = run_once(&cli.files, &settings, !cli.no_cache, cli.fix)?;
         if !cli.quiet {
-            report_once(&messages)?;
+            printer.write_once(&messages)?;
         }
 
         #[cfg(feature = "update-informer")]
