@@ -7,25 +7,10 @@ use serde::{Deserialize, Serialize};
 
 use crate::checks::CheckKind;
 
-#[derive(Serialize, Deserialize)]
-#[serde(remote = "Location")]
-struct LocationDef {
-    #[serde(getter = "Location::row")]
-    row: usize,
-    #[serde(getter = "Location::column")]
-    column: usize,
-}
-
-impl From<LocationDef> for Location {
-    fn from(def: LocationDef) -> Location {
-        Location::new(def.row, def.column)
-    }
-}
-
 #[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
 pub struct Message {
     pub kind: CheckKind,
-    #[serde(with = "LocationDef")]
+    pub fixed: bool,
     pub location: Location,
     pub filename: String,
 }
@@ -35,7 +20,7 @@ impl Ord for Message {
         (&self.filename, self.location.row(), self.location.column()).cmp(&(
             &other.filename,
             other.location.row(),
-            self.location.column(),
+            other.location.column(),
         ))
     }
 }
